from typing import Any, Awaitable, Coroutine, cast, Union, Dict
import warnings

import grpc
from google.protobuf.json_format import ParseDict
from pycityproto.city.person.v1 import person_pb2 as person_pb2
from pycityproto.city.person.v1 import person_service_pb2 as person_service
from pycityproto.city.person.v1 import person_service_pb2_grpc as person_grpc

from ..utils.protobuf import async_parse

__all__ = ["PersonService"]


class PersonService:
    """
    交通模拟person服务
    Traffic simulation person service
    """

    def __init__(self, aio_channel: grpc.aio.Channel):
        self._aio_stub = person_grpc.PersonServiceStub(aio_channel)

    @staticmethod
    def default_agent() -> person_pb2.Person:
        warnings.warn(
            "default_agent is deprecated, use default_person instead",
            DeprecationWarning,
        )
        return PersonService.default_person()

    @staticmethod
    def default_person() -> person_pb2.Person:
        """
        获取person基本模板
        Get person basic template

        需要补充的字段有person.home,person.schedules,person.labels
        The fields that need to be supplemented are person.home, person.schedules, person.labels
        """
        person = person_pb2.Person(
            attribute=person_pb2.PersonAttribute(
                length=5.0,
                width=2.0,
                max_speed=41.6666666667,
                max_acceleration=3.0,
                max_braking_acceleration=-10.0,
                usual_acceleration=2.0,
                usual_braking_acceleration=-4.5,
            ),
            vehicle_attribute=person_pb2.VehicleAttribute(
                lane_change_length=10.0, min_gap=1.0
            ),
        )
        return person

    def GetPerson(
        self,
        req: Union[person_service.GetPersonRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], person_service.GetPersonResponse]]:
        """
        获取person信息
        Get person information

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.person.1.GetPersonRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.person.1.GetPersonResponse
        """
        if type(req) != person_service.GetPersonRequest:
            req = ParseDict(req, person_service.GetPersonRequest())
        res = cast(
            Awaitable[person_service.GetPersonResponse], self._aio_stub.GetPerson(req)
        )
        return async_parse(res, dict_return)

    def AddPerson(
        self,
        req: Union[person_service.AddPersonRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], person_service.AddPersonResponse]]:
        """
        新增person
        Add a new person

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.person.1.AddPersonRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.person.1.AddPersonResponse
        """
        if type(req) != person_service.AddPersonRequest:
            req = ParseDict(req, person_service.AddPersonRequest())
        res = cast(
            Awaitable[person_service.AddPersonResponse], self._aio_stub.AddPerson(req)
        )
        return async_parse(res, dict_return)

    def SetSchedule(
        self,
        req: Union[person_service.SetScheduleRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], person_service.SetScheduleResponse]]:
        """
        修改person的schedule
        set person's schedule

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.person.1.SetScheduleRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.person.1.SetScheduleResponse
        """
        if type(req) != person_service.SetScheduleRequest:
            req = ParseDict(req, person_service.SetScheduleRequest())
        res = cast(
            Awaitable[person_service.SetScheduleResponse],
            self._aio_stub.SetSchedule(req),
        )
        return async_parse(res, dict_return)

    def GetPersonByLongLatBBox(
        self,
        req: Union[person_service.GetPersonByLongLatBBoxRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[
        Any, Any, Union[Dict[str, Any], person_service.GetPersonByLongLatBBoxResponse]
    ]:
        """
        获取特定区域内的person
        Get persons in a specific region

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.person.1.GetPersonByLongLatBBoxRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.person.1.GetPersonByLongLatBBoxResponse
        """
        if type(req) != person_service.GetPersonByLongLatBBoxRequest:
            req = ParseDict(req, person_service.GetPersonByLongLatBBoxRequest())
        res = cast(
            Awaitable[person_service.GetPersonByLongLatBBoxResponse],
            self._aio_stub.GetPersonByLongLatBBox(req),
        )
        return async_parse(res, dict_return)



from typing import Any, Awaitable, Coroutine, cast, Union, Dict

import grpc
from google.protobuf.json_format import ParseDict
from pycityproto.city.map.v2 import road_service_pb2 as road_service
from pycityproto.city.map.v2 import road_service_pb2_grpc as road_grpc

from ..utils.protobuf import async_parse

__all__ = ["RoadService"]


class RoadService:
    """
    交通模拟road服务
    Traffic simulation road service
    """

    def __init__(self, aio_channel: grpc.aio.Channel):
        self._aio_stub = road_grpc.RoadServiceStub(aio_channel)

    def GetRoad(
        self, req: Union[road_service.GetRoadRequest, dict], dict_return: bool = True
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], road_service.GetRoadResponse]]:
        """
        查询道路信息
        Query road information
        
        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.map.v2.GetRoadRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.map.v2.GetRoadResponse
        """
        if type(req) != road_service.GetRoadRequest:
            req = ParseDict(req, road_service.GetRoadRequest())
        res = cast(Awaitable[road_service.GetRoadResponse], self._aio_stub.GetRoad(req))
        return async_parse(res, dict_return)



from typing import Any, Awaitable, Coroutine, cast, Union, Dict

from google.protobuf.json_format import ParseDict
from pycityproto.city.routing.v2 import routing_service_pb2 as routing_service
from pycityproto.city.routing.v2 import routing_service_pb2_grpc as routing_grpc

from ..utils.protobuf import async_parse
from ..utils.grpc import create_aio_channel

__all__ = [
    "RoutingClient",
]


class RoutingClient:
    """
    Routing服务的client端
    Client side of Routing service
    """

    def __init__(self, server_address: str, secure: bool = False):
        """
        RoutingClient的构造函数
        Constructor of RoutingClient

        Args:
        - server_address (str): routing服务器地址。Routing server address
        - secure (bool, optional): 是否使用安全连接. Defaults to False. Whether to use a secure connection. Defaults to False.
        """
        aio_channel = create_aio_channel(server_address, secure)
        self._aio_stub = routing_grpc.RoutingServiceStub(aio_channel)

    def GetRoute(
        self,
        req: Union[routing_service.GetRouteRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], routing_service.GetRouteResponse]]:
        """
        请求导航
        Request navigation

        Args:
        - req (routing_service.GetRouteRequest): https://cityproto.sim.fiblab.net/#city.routing.v2.GetRouteRequest
        - dict_return (bool, optional): 是否返回dict类型的结果. Defaults to True. Whether to return a dict type result. Defaults to True.

        Returns:
        - https://cityproto.sim.fiblab.net/#city.routing.v2.GetRouteResponse
        """
        if type(req) != routing_service.GetRouteRequest:
            req = ParseDict(req, routing_service.GetRouteRequest())
        res = cast(
            Awaitable[routing_service.GetRouteResponse], self._aio_stub.GetRoute(req)
        )
        return async_parse(res, dict_return)



from typing import Any, Awaitable, Coroutine, cast, Union, Dict

import grpc
from google.protobuf.json_format import ParseDict
from pycityproto.city.map.v2 import traffic_light_service_pb2 as light_service
from pycityproto.city.map.v2 import traffic_light_service_pb2_grpc as light_grpc

from ..utils.protobuf import async_parse

__all__ = ["LightService"]


class LightService:
    """
    城市模拟信控服务
    City simulation traffic light service
    """

    def __init__(self, aio_channel: grpc.aio.Channel):
        self._aio_stub = light_grpc.TrafficLightServiceStub(aio_channel)

    def GetTrafficLight(
        self,
        req: Union[light_service.GetTrafficLightRequest, Dict[str, Any]],
        dict_return: bool = True,
    ) -> Coroutine[
        Any, Any, Union[Dict[str, Any], light_service.GetTrafficLightResponse]
    ]:
        """
        获取路口的红绿灯信息
        Get traffic light information

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.traffic.v1.GetTrafficLightRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.traffic.v1.GetTrafficLightResponse
        """
        if type(req) != light_service.GetTrafficLightRequest:
            req = ParseDict(req, light_service.GetTrafficLightRequest())
        res = cast(
            Awaitable[light_service.GetTrafficLightResponse],
            self._aio_stub.GetTrafficLight(req),
        )
        return async_parse(res, dict_return)

    def SetTrafficLight(
        self,
        req: Union[light_service.SetTrafficLightRequest, Dict[str, Any]],
        dict_return: bool = True,
    ) -> Coroutine[
        Any, Any, Union[Dict[str, Any], light_service.SetTrafficLightResponse]
    ]:
        """
        设置路口的红绿灯信息
        Set traffic light information

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.traffic.v1.SetTrafficLightRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.traffic.v1.SetTrafficLightResponse
        """
        if type(req) != light_service.SetTrafficLightRequest:
            req = ParseDict(req, light_service.SetTrafficLightRequest())
        res = cast(
            Awaitable[light_service.SetTrafficLightResponse],
            self._aio_stub.SetTrafficLight(req),
        )
        return async_parse(res, dict_return)

    def SetTrafficLightPhase(
        self,
        req: Union[light_service.SetTrafficLightPhaseRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[
        Any, Any, Union[Dict[str, Any], light_service.SetTrafficLightPhaseResponse]
    ]:
        """
        设置路口的红绿灯相位
        Set traffic light phase

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.traffic.v1.SetTrafficLightPhaseRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.traffic.v1.SetTrafficLightPhaseResponse
        """
        if type(req) != light_service.SetTrafficLightPhaseRequest:
            req = ParseDict(req, light_service.SetTrafficLightPhaseRequest())
        res = cast(
            Awaitable[light_service.SetTrafficLightPhaseResponse],
            self._aio_stub.SetTrafficLightPhase(req),
        )
        return async_parse(res, dict_return)

    def SetTrafficLightStatus(
        self,
        req: Union[light_service.SetTrafficLightStatusRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[
        Any, Any, Union[Dict[str, Any], light_service.SetTrafficLightStatusResponse]
    ]:
        """
        设置路口的红绿灯状态
        Set traffic light status

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.traffic.v1.SetTrafficLightStatusRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.traffic.v1.SetTrafficLightStatusResponse
        """
        if type(req) != light_service.SetTrafficLightStatusRequest:
            req = ParseDict(req, light_service.SetTrafficLightStatusRequest())
        res = cast(
            Awaitable[light_service.SetTrafficLightStatusResponse],
            self._aio_stub.SetTrafficLightStatus(req),
        )
        return async_parse(res, dict_return)



from typing import List

__all__ = ["wrap_feature_collection"]


def wrap_feature_collection(features: List[dict], name: str):
    """
    将 GeoJSON Feature 集合包装为 FeatureCollection
    Wrap GeoJSON Feature collection as FeatureCollection

    Args:
    - features: GeoJSON Feature 集合。GeoJSON Feature collection.
    - name: FeatureCollection 名称。FeatureCollection name.

    Returns:
    - dict: GeoJSON FeatureCollection
    """
    return {
        "type": "FeatureCollection",
        "name": name,
        "crs": {
            "type": "name",
            "properties": {"name": "urn:ogc:def:crs:OGC:1.3:CRS84"},
        },
        "features": features,
    }



import warnings
import grpc

from ..sidecar import OnlyClientSidecar
from .clock_service import ClockService
from .person_service import PersonService
from .aoi_service import AoiService
from .lane_service import LaneService
from .road_service import RoadService
from .social_service import SocialService
from .economy_services import EconomyPersonService, EconomyOrgService
from .event_service import EventService
from .light_service import LightService

from ..utils.grpc import create_aio_channel

__all__ = ["CityClient"]


class CityClient:
    """
    模拟器接口
    Simulator interface
    """

    NAME = "city"

    def __init__(
        self,
        url: str,
        secure: bool = False,
    ):
        """
        Args:
        - url (str): 模拟器server的地址。The address of the emulator server.
        - secure (bool, optional): 是否使用安全连接. Defaults to False. Whether to use a secure connection. Defaults to False.
        """
        aio_channel = create_aio_channel(url, secure)
        self._clock_service = ClockService(aio_channel)
        self._lane_service = LaneService(aio_channel)
        self._person_service = PersonService(aio_channel)
        self._aoi_service = AoiService(aio_channel)
        self._road_service = RoadService(aio_channel)
        self._social_service = SocialService(aio_channel)
        self._economy_person_service = EconomyPersonService(aio_channel)
        self._economy_org_service = EconomyOrgService(aio_channel)
        self._event_service = EventService(aio_channel)
        self._light_service = LightService(aio_channel)

    @staticmethod
    def from_sidecar(sidecar: OnlyClientSidecar, name: str = NAME):
        """
        从sidecar中创建CityClient
        Create CityClient from sidecar
        """
        return CityClient(sidecar.wait_url(name))

    @property
    def clock_service(self):
        """
        模拟器时间服务子模块
        Simulator time service submodule
        """
        return self._clock_service

    @property
    def lane_service(self):
        """
        模拟器lane服务子模块
        Simulator lane service submodule
        """
        return self._lane_service

    @property
    def agent_service(self):
        """
        模拟器智能体服务子模块（已弃用，请使用.person_service）
        Simulator agent service submodule (deprecated, please use .person_service)
        """
        warnings.warn(
            "agent_service is deprecated, use person_service instead",
            DeprecationWarning,
        )
        return self._person_service

    @property
    def person_service(self):
        """
        模拟器智能体服务子模块
        Simulator agent service submodule
        """
        return self._person_service

    @property
    def aoi_service(self):
        """
        模拟器AOI服务子模块
        Simulator AOI service submodule
        """
        return self._aoi_service

    @property
    def road_service(self):
        """
        模拟器road服务子模块
        Simulator road service submodule
        """
        return self._road_service

    @property
    def social_service(self):
        """
        模拟器social服务子模块
        Simulator social service submodule
        """
        return self._social_service

    @property
    def economy_person_service(self):
        """
        模拟器经济服务（个人）子模块
        Simulator economic service (personal) submodule
        """
        return self._economy_person_service

    @property
    def economy_org_service(self):
        """
        模拟器经济服务（组织）子模块
        Simulator economic service (organizational) submodule
        """
        return self._economy_org_service

    @property
    def event_service(self):
        """
        模拟器事件服务子模块
        Simulator event service submodule
        """
        return self._event_service

    @property
    def light_service(self):
        """
        模拟器红绿灯服务子模块
        Simulator traffic light service submodule
        """
        return self._light_service
